'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var vue = _interopDefault(require('vue'));

var originalContent = undefined;
/**
 * Default color of highlighted texts
 * @type {string}
 */
var textColorDefault = '#fff';

/**
 * Default background color of highlighted texts
 * @type {string}
 */
var bgColorDefault = '#009688';

/**
 * Default padding of highlight containers
 * @type {string}
 */
var paddingDefault = '0px 5px';

/**
 * Create a RegEx pattern to find the desired words
 * @param {(string|string[])} word - A string or an array of strings to use to create the RegEx pattern
 * @param {boolean} [liveHighlighting=false] - Specify if the pattern should be a live search. Default is false
 * @returns {string} The RegEx pattern to find looked for words
 */
function selectPattern(word, liveHighlighting) {
  if ( liveHighlighting === void 0 ) liveHighlighting = false;

  var result = "";
  var explodedWord = Array.isArray(word) ? word.join('|') : word;
  if (!liveHighlighting) {
    result = "\\b(" + explodedWord + ")\\b";
  } else {
    result = "(" + explodedWord + ")";
  }
  return result;
}

/**
 * Set text color of highlighted text
 * @param {string} [color=textColorDefault] - See {@link testColor} for valid colors
 * @returns {string} Return the passed string if it is valid color, the {@link textColorDefault} otherwise
 */
function testTextColor(color) {
  if ( color === void 0 ) color = textColorDefault;

  var result = textColorDefault;
  if (testColor(color)) {
    result = color;
  }
  return result;
}

/**
 * Set text color of highlighted text
 * @param {string} [color=bgColorDefault] - See {@link testColor} for valid colors
 * @returns {string} Return the passed string if it is valid color, the {@link bgColorDefault} otherwise
 */
function testBgColor(color) {
  if ( color === void 0 ) color = bgColorDefault;

  var result = bgColorDefault;
  if (testColor(color)) {
    result = color;
  }
  return result;
}

/**
 * Test if a string representing a color is valid
 * @param {string} color - Color to test. Valid color are default CSS text colors (e.g.: 'black') and 3 or 6 hexadecimals preceded by a #
 * @returns {boolean} True if the color is considered valid, false otherwise
 */
function testColor(color) {
  var result = false;
  var isAColor = /(^#[0-9a-zA-F]{8}$)|(^#[0-9a-zA-F]{6}$)|(^#[0-a-z9A-F]{4}$)|(^#[0-9a-zA-F]{3}$)/i.test(color);
  var isAString = /^[a-zA-Z]+$/.test(color);
  if (isAColor || isAString) {
    result = true;
  }
  return result;
}

/**
 * Test if the passed padding value is valid
 * @param {string} padding - The padding value to test
 * @returns {string} The passed padding if it is valid, {@link paddingDefault} otherwise
 */
function testPadding(padding) {
  var result = paddingDefault;
  var isAValidValue = /^(\d+(cm|mm|in|px|pt|pc|em|ex|ch|rem|vw|vh|vmin|vmax|%)\s?){1,4}$/i.test(padding);
  if (isAValidValue) {
    result = padding;
  }
  return result;
}

/**
 * Create tags with highlighted looked for text
 * @param {string} content - Content where find text to highlight
 * @param {(string|string[])} word - A word or an array of words to looking for
 * @param {string} patternSelected - RegEx pattern to find looked for words
 * @param {string} color - Text color of highlighted container
 * @param {string} bgColor - Background color of highlighted container
 * @param {string} padding - Padding of highlighted container
 * @returns {string} Return replaced content with highlighted tags
 */
function highlight(content, word, patternSelected, color, bgColor, padding) {
  var spanStart = "<span style='padding:" + padding + "; background-color:" + bgColor + "; color:" + color + ";'>";
  var spanEnd = "</span>";
  var result = content;
  if (word != "") {
    var regex = new RegExp(patternSelected, "gi");
    result = content.replace(regex, spanStart + "$&" + spanEnd);
  }
  return result;
}

var vueHighlighter = {
  bind: function bind(el, binding, vnode) {
    originalContent = el.innerHTML;
    var pattern = '';
    var word = '';
    var color = textColorDefault;
    var bgColor = bgColorDefault;
    var padding = paddingDefault;
    if (binding.value.word != undefined) {
      word = binding.value.word;
    }
    if (binding.value.live != undefined) {
      pattern = selectPattern(word, binding.value.live);
    }
    if (binding.value.style != undefined) {
      color = testTextColor(binding.value.style.color);
      bgColor = testBgColor(binding.value.style.bgColor);
      padding = testPadding(binding.value.style.padding);
    }
    el.innerHTML = highlight(originalContent, word, pattern, color, bgColor, padding);
  },
  update: function update(el, binding, vnode, oldVnode) {
    var pattern = '';
    var color = textColorDefault;
    var bgColor = bgColorDefault;
    var padding = paddingDefault;
    if (binding.value.style != undefined) {
      color = testTextColor(binding.value.style.color);
      bgColor = testBgColor(binding.value.style.bgColor);
      padding = testPadding(binding.value.style.padding);
    }
    if (binding.value.live) {
      pattern = selectPattern(binding.value.word, binding.value.live);
      el.innerHTML = highlight(vnode.children[0].text, binding.value.word, pattern, color, bgColor, padding);
    } else {
      pattern = selectPattern(binding.value.word);
      el.innerHTML = highlight(originalContent, binding.value.word, pattern, color, bgColor, padding);
    }
  },
  unbind: function unbind(el, binding, vnode) {
    el.innerHTML = originalContent;
  }
};

var Plugin = {
  install: function install(Vue, options) {
    Vue.directive('highlight', vueHighlighter);
  }
};

// Install by default
if (typeof window !== 'undefined' && window.Vue) {
  window.Vue.use(Plugin);
}

exports['default'] = Plugin;
exports.highlight = vueHighlighter;
